
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#define COMPILE_TEMPLATE_ALGORITHM_FACET_CLASSES
#include "algorithm_facet_classes.h"
#undef COMPILE_TEMPLATE_ALGORITHM_FACET_CLASSES

#include <algorithm>
#include <cassert>
#include <cstddef>
#include <set>
#include <tuple>
#include <vector>

#include "algorithm_map_operations.h"
#include "algorithm_row_operations.h"

template <typename Integer>
Facet<Integer> algorithm::classRepresentative(const Facet<Integer>& facet,
                                              const Maps& maps)
{
   const auto facets = facetClass(facet, maps);
   assert( !facets.empty() );
   return *facets.crbegin();
}

template <typename Integer>
std::set<Facet<Integer>> algorithm::facetClass(const Facet<Integer>& facet, const Maps& maps)
{
   std::set<Facet<Integer>> facets;
   facets.insert(facet);
   using Iterator = typename std::set<Facet<Integer>>::iterator;
   std::vector<Iterator> iterators;
   iterators.push_back(facets.begin());
   while ( !iterators.empty() )
   {
      const auto& current_facet = *iterators.back();
      iterators.pop_back();
      for ( const auto& map : maps )
      {
         const auto new_facet = applyOnInequality(map, current_facet);
         Iterator iterator;
         bool inserted;
         std::tie(iterator, inserted) = facets.insert(new_facet);
         if ( inserted )
         {
            iterators.push_back(iterator);
         }
      }
   }
   return facets;
}

template <typename Integer>
Facets<Integer> algorithm::facetClasses(Facets<Integer> fs, const Maps& maps)
{
   std::set<Facet<Integer>> facets(fs.cbegin(), fs.cend());
   return facetClasses(facets, maps);
}

template <typename Integer>
Facets<Integer> algorithm::facetClasses(std::set<Facet<Integer>> facets, const Maps& maps)
{
   Facets<Integer> classes;
   while ( !facets.empty() )
   {
      const auto facet_class = facetClass(*facets.begin(), maps);
      assert( !facet_class.empty() );
      classes.push_back(*facet_class.crbegin());
      for ( const auto& facet : facet_class )
      {
         const auto position = facets.find(facet);
         if ( position != facets.end() )
         {
            facets.erase(position);
         }
      }
   }
   return classes;
}

