
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#pragma once

#include <set>

#include "maps.h"
#include "matrix.h"
#include "row.h"

namespace algorithm
{
   /// returns a unique class representative. The class is generated with the provided maps.
   /// Precondition: input facet must be normalized.
   template <typename Integer>
   Facet<Integer> classRepresentative(const Facet<Integer>&, const Maps&);
   template <typename Integer>
   /// Creates a set of facets which is the complete facet class containing the input facet.
   /// Precondition: input facet must be normalized.
   std::set<Facet<Integer>> facetClass(const Facet<Integer>&, const Maps&);
   /// Reduces a list of facets to just the representatives.
   /// Precondition: input facets must be normalized.
   template <typename Integer>
   Facets<Integer> facetClasses(Facets<Integer>, const Maps&);
   /// Reduces a list of facets to just the representatives.
   /// Precondition: input facets must be normalized.
   template <typename Integer>
   Facets<Integer> facetClasses(std::set<Facet<Integer>>, const Maps&);
}

#include "algorithm_facet_classes.eti"

