
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#define COMPILE_TEMPLATE_ALGORITHM_INTEGER_OPERATIONS
#include "algorithm_integer_operations.h"
#undef COMPILE_TEMPLATE_ALGORITHM_INTEGER_OPERATIONS

#include <cassert>
#include <cstdlib>
#include <cstdint>

namespace
{
   /// optimized gcd for unsigned integer values.
   template <typename Integer>
   Integer unsigned_gcd(Integer, Integer);
}

template <typename Integer>
Integer gcd(Integer a, Integer b)
{
   using std::abs;
   return unsigned_gcd(abs(a), abs(b));
}

template <typename Integer>
Integer lcm(Integer a, Integer b)
{
   const auto gcd_value = gcd(a, b);
   using std::abs;
   return abs(a) * (abs(b) / gcd_value);
}

namespace
{
   template <typename Integer>
   Integer unsigned_gcd(Integer a, Integer b)
   {
      assert( a >= 0 && b >= 0 );
      while ( a != 0 && b != 0 )
      {
         if ( a > b )
         {
            a %= b;
         }
         else
         {
            b %= a;
         }
      }
      return (a == 0) ? b : a;
   }
}

