
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#define COMPILE_TEMPLATE_ALGORITHM_POLYTOPE_VALIDATION
#include "algorithm_polytope_validation.h"
#undef COMPILE_TEMPLATE_ALGORITHM_POLYTOPE_VALIDATION

#include <iostream>
#include <sstream>

#include "algorithm_inequality_operations.h"
#include "algorithm_row_operations.h"
#include "row.h"

namespace
{
   /// checks validity of an inequality.
   template <typename Integer>
   void inequalityValidation(const ConvexHull<Integer>&, const ConicalHull<Integer>&, const Inequality<Integer>&);
}

template <typename Integer>
void algorithm::polytopeValidation(const Polytope<Integer>& polytope)
{
   const ConvexHull<Integer>& conv = std::get<0>(polytope);
   const ConicalHull<Integer>& cone = std::get<1>(polytope);
   const Inequalities<Integer>& ieqs = std::get<2>(polytope);
   for ( const auto& inequality : ieqs )
   {
      inequalityValidation(conv, cone, inequality);
   }
}
namespace
{
   template <typename Integer>
   void validation(const Matrix<Integer>& matrix, const Inequality<Integer>& inequality, const char* type)
   {
      for ( const auto& row : matrix )
      {
         if ( distance(inequality, row) < 0 )
         {
            std::stringstream stream;
            stream << "Inequality (" << inequality << ") is violated by " << type << " (" << row << ")\n";
            std::cerr << stream.str();
            return;
         }
      }
   }

   template <typename Integer>
   void inequalityValidation(const ConvexHull<Integer>& conv,
                             const ConicalHull<Integer>& cone,
                             const Inequality<Integer>& inequality)
   {
      validation(conv, inequality, "vertex");
      validation(cone, inequality, "ray");
   }
}

