
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "big_integer.h"

#include <cstdlib>
#include <stdexcept>

BigInteger::BigInteger()
:
   sign(Sign::Positive),
   data(1, 0)
{
}

#ifdef INT16_MAX
BigInteger::BigInteger(const int16_t value)
:
   sign((value >= 0) ? Sign::Positive : Sign::Negative),
   data(1, (value >= 0) ? value : -value)
{
}
#endif

#ifdef INT32_MAX
BigInteger::BigInteger(const int32_t value)
:
   sign((value >= 0) ? Sign::Positive : Sign::Negative),
   data(1, std::abs(value))
{
}
#endif

#ifdef INT64_MAX
BigInteger::BigInteger(const int64_t value)
:
   sign((value >= 0) ? Sign::Positive : Sign::Negative),
   data(1, std::abs(value))
{
}
#endif

#ifdef UINT16_MAX
BigInteger::BigInteger(const uint16_t value)
:
   sign(Sign::Positive),
   data(1, value)
{
}
#endif

#ifdef UINT32_MAX
BigInteger::BigInteger(const uint32_t value)
:
   sign(Sign::Positive),
   data(1, value)
{
}
#endif

#ifdef UINT64_MAX
BigInteger::BigInteger(const uint64_t value)
:
   sign(Sign::Positive),
   data(1, value)
{
}
#endif

BigInteger::operator int() const
{
   if ( data.size() > 1 )
   {
      throw std::logic_error("Number doesn't fit into int.");
   }
   if ( sign == Sign::Positive )
   {
      const int value = data[0];
      if ( value == 0 && data[0] != 0 )
      {
         throw std::logic_error("Number doesn't fit into int.");
      }
      return value;
   }
   else
   {
      const int value = data[0];
      if ( value == 0 && data[0] != 0 )
      {
         throw std::logic_error("Number doesn't fit into int.");
      }
      return -value;
   }
}

