
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#define COMPILE_TEMPLATE_BITSET_FIXED_SIZE
#include "bitset_fixed_size.h"
#undef COMPILE_TEMPLATE_BITSET_FIXED_SIZE

#include <algorithm>
#include <cassert>
#include <limits>

#include "popcount.h"

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
template <std::size_t Size>
BitsetFixedSize<Size>::BitsetFixedSize(const std::size_t b)
:
   data()
{
   assert( b <= std::numeric_limits<DataType>::digits * Size );
}
#pragma GCC diagnostic pop

template <std::size_t Size>
bool BitsetFixedSize<Size>::equals(const BitsetFixedSize<Size>& second, const std::size_t max) const
{
   const auto end = data.cbegin() + 1 + (max - 1) / std::numeric_limits<DataType>::digits;
   return std::equal(data.cbegin(), end, second.data.cbegin());
}

template <std::size_t Size>
bool BitsetFixedSize<Size>::contains(const BitsetFixedSize<Size>& second, const std::size_t max) const
{
   const auto end = 1u + (max - 1) / std::numeric_limits<DataType>::digits;
   for ( std::size_t i(0); i < end; ++i )
   {
      if ( (data[i] & second.data[i]) != second.data[i] )
      {
         return false;
      }
   }
   return true;
}

template <std::size_t Size>
std::size_t BitsetFixedSize<Size>::count(const std::size_t max) const
{
   std::size_t total{0};
   const auto end = data.cbegin() + 1 + (max - 1) / std::numeric_limits<DataType>::digits;
   for ( auto it(data.cbegin()); it != end; ++it )
   {
      total += popcount(*it);
   }
   return total;
}

template <std::size_t Size>
BitsetFixedSize<Size> BitsetFixedSize<Size>::merge(const BitsetFixedSize<Size>& second, const std::size_t max) const
{
   auto result = *this;
   const auto end = 1 + (max - 1) / std::numeric_limits<DataType>::digits;
   for ( std::size_t i(0); i < end; ++i )
   {
      result.data[i] |= second.data[i];
   }
   return result;
}

template <std::size_t Size>
void BitsetFixedSize<Size>::set(const std::size_t index)
{
   const auto mask = static_cast<DataType>(1u) << (index % std::numeric_limits<DataType>::digits);
   assert(index / std::numeric_limits<DataType>::digits < Size );
   data[index / std::numeric_limits<DataType>::digits] |= mask;
}

