
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#pragma once

#include <array>
#include <cstddef>
#include <cstdint>

template <std::size_t Size>
class BitsetFixedSize
{
   public:
      /// Underlying data type.
      using DataType = uint32_t;
      /// Constructor: argument denotes number of bits.
      BitsetFixedSize(const std::size_t);
      /// Default copy constructor.
      BitsetFixedSize(const BitsetFixedSize&) = default;
      /// Default move constructor.
      BitsetFixedSize(BitsetFixedSize&&) = default;
      /// Default copy assignment operator.
      BitsetFixedSize& operator=(const BitsetFixedSize&) = default;
      /// Default move assignment operator.
      BitsetFixedSize& operator=(BitsetFixedSize&&) = default;
      /// Comparison (equality) with another Bitset with a hint of highest set bit.
      bool equals(const BitsetFixedSize<Size>&, const std::size_t) const;
      /// Checks if the passed Bitset is contained in this (with hint of highest set bit).
      bool contains(const BitsetFixedSize<Size>&, const std::size_t) const;
      /// Returns the union with a second bitset with a hint of highest set bit.
      BitsetFixedSize<Size> merge(const BitsetFixedSize<Size>&, const std::size_t) const;
      /// Returns the number of 1s in the bitset with a hint of highest set bit.
      std::size_t count(const std::size_t) const;
      /// Sets the i^th bit.
      void set(const std::size_t);
   private:
      std::array<DataType, Size> data;
};

#include "bitset_fixed_size.eti"

