
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#pragma once

#include <condition_variable>
#include <cstddef>
#include <mutex>
#include <set>
#include <vector>

#include "matrix.h"
#include "names.h"
#include "row.h"

template <typename Integer>
class FacetList
{
   public:
      /// merges facets with the list of facets held in the list.
      void put(const Facets<Integer>&) const;
      /// merges a facet with the list of facets held in the list.
      void put(const Facet<Integer>&) const;
      /// Returns a facet that wasn't ever returned here before. Blocks the
      /// caller until data is available.
      Facet<Integer> get() const;
      #pragma GCC diagnostic push
      #pragma GCC diagnostic ignored "-Weffc++"
      /// Constructor: special thing here: number of active workers is initialized
      /// to 1 (allowing heuristic to fill in once).
      FacetList(const Names&);
      #pragma GCC diagnostic pop
   private:
      const Names names;
      mutable std::mutex mutex;
      mutable std::size_t workers;
      mutable std::condition_variable condition;
      mutable std::set<Facet<Integer>> facets;
      using Iterator = typename std::set<Facet<Integer>>::iterator;
      mutable std::vector<Iterator> iterators;
      mutable std::size_t counter;
   private:
      /// checks if all jobs are done.
      bool empty() const;
};

#include "facet_list.eti"

