
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#pragma once

#include <tuple>
#include <utility>

#include "maps.h"
#include "matrix.h"
#include "names.h"
#include "polytope.h"
#include "row.h"

namespace input
{
   /// Reads a conical/convex hull description with optional names and maps.
   template <typename Integer>
   std::tuple<Vertices<Integer>, Names, Maps> vertices(int, char**);

   /// Reads a inequality description.
   template <typename Integer>
   Inequalities<Integer> inequalities(int, char**);

   /// Reads additional information (known facets) for the Adjacency Decomposition.
   template <typename Integer>
   Inequalities<Integer> knownFacets(int, char**, const Names&);

   /// Reads a polytope (conv/cone/ieq description).
   template <typename Integer>
   Polytope<Integer> polytope(int, char**);
}

// explicit template instantiations
namespace input
{
   template <>
   std::tuple<Vertices<int>, Names, Maps> vertices<int>(int, char**);

   template <>
   Inequalities<int> inequalities<int>(int, char**);

   template <>
   Inequalities<int> knownFacets<int>(int, char**, const Names&);

   template <>
   Polytope<int> polytope<int>(int, char**);
}

#include "input.tpp"

