
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "input_inequality.h"

#include <stdexcept>
#include <string>
#include <tuple>

#include "algorithm_row_operations.h"
#include "constraint_type.h"
#include "input_common.h"
#include "input_inequality_named_variables.h"
#include "input_inequality_unnamed_variables.h"
#include "input_keywords.h"
#include "istream_peek_line.h"

namespace
{
   std::pair<Inequality<int>, ConstraintType> inequalityPanda(std::istream&, const Names&);
}

Inequalities<int> input::implementation::inequalitiesPanda(std::istream& stream, const Names& names)
{
   Inequalities<int> ieqs;
   std::string token;
   if ( !(stream >> token) || !isKeywordInequalities(token) )
   {
      throw std::invalid_argument("Cannot read inequalities: file is at an invalid position.");
   }
   while ( stream && input::firstWord(peekLine(stream)) != "END" )
   {
      Inequality<int> ieq;
      ConstraintType type;
      std::tie(ieq, type) = inequalityPanda(stream, names);
      if ( !ieq.empty() )
      {
         ieqs.push_back(ieq);
         if ( type == ConstraintType::Equality )
         {
            ieqs.push_back(-1 * ieq);
         }
      }
      while ( stream && stream.peek() != '\n' )
      {
         stream.get();
      }
      stream.get();
   }
   if ( stream )
   {
      std::getline(stream, token); // consume "END".
   }
   return ieqs;
}

namespace
{
   std::pair<Inequality<int>, ConstraintType> inequalityPanda(std::istream& stream, const Names& names)
   {
      if ( names.empty() || !input::containsNames(names, stream) )
      {
         return input::implementation::inequality(stream);
      }
      else
      {
         return input::implementation::inequality(stream, names);
      }
   }
}

