
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "input_keywords.h"

#include "input_common.h"
#include "position_guarded_file.h"

bool input::implementation::containsKeyword(const PositionGuardedFile& pg, Strings keywords)
{
   for ( std::string line; std::getline(pg.file, line); )
   {
      line = input::firstWord(line);
      for ( const auto keyword : keywords )
      {
         if ( line == keyword )
         {
            return true;
         }
      }
   }
   return false;
}

bool input::implementation::containsKeywords(const PositionGuardedFile& pg)
{
   // We are not going to read the whole file, because it is potentially huge.
   // Instead, we know that in no sane file, the first 20 lines are free of keywords.
   // This is because the lines would have to be empty.
   const int max_tokens = 20;
   int i = 0;
   for ( std::string token; i < max_tokens && std::getline(pg.file, token); ++i )
   {
      token = input::firstWord(token);
      if ( input::implementation::isKeyword(token) )
      {
         return true;
      }
   }
   return false;
}


bool input::implementation::isKeyword(const std::string& string)
{
   return (isKeywordConicalHull(string) ||
           isKeywordConvexHull(string) ||
           isKeywordDimension(string) ||
           isKeywordInequalities(string) ||
           isKeywordMaps(string) ||
           isKeywordNames(string) ||
           string == "END");
}

bool input::implementation::isKeywordConicalHull(const std::string& string)
{
   return (string == "CONE_SECTION" ||
           string == "CONICAL_HULL" ||
           string == "CONICAL_HULL_SECTION");
}

bool input::implementation::isKeywordConvexHull(const std::string& string)
{
   return (string == "CONVEX_HULL" ||
           string == "CONVEX_HULL_SECTION" ||
           string == "CONV_SECTION");
}

bool input::implementation::isKeywordDimension(const std::string& string)
{
   return (string == "DIM" ||
           string == "DIM=" ||
           string == "DIMENSION" ||
           string == "DIMENSION=");
}

bool input::implementation::isKeywordInequalities(const std::string& string)
{
   return (string == "INEQUALITIES" ||
           string == "INEQUALITY_SECTION" ||
           string == "INEQUALITIES_SECTION" );
}

bool input::implementation::isKeywordMaps(const std::string& string)
{
   return (string == "MAPS" ||
           string == "MAP_SECTION" ||
           string == "MAPS_SECTION" ||
           string == "PERMUTATIONS" ||
           string == "PERMUTATION_SECTION" ||
           string == "PERMUTATIONS_SECTION");
}

bool input::implementation::isKeywordNames(const std::string& string)
{
   return (string == "INDEX" ||
           string == "INDICES" ||
           string == "INDEX_SECTION" ||
           string == "INDICES_SECTION" ||
           string == "NAMES" ||
           string == "NAME_SECTION" ||
           string == "NAMES_SECTION");
}

