
#define COMPILE_TEMPLATE_JOB_MANAGER
#include "job_manager.h"
#undef COMPILE_TEMPLATE_JOB_MANAGER

//#define BENCHMARK_LOAD_BALANCING

#include <iostream>
#include <sstream>

#include "algorithm_row_operations.h"

template <typename Integer>
void JobManager<Integer>::put(const Matrix<Integer>& matrix) const
{
   facets.put(matrix);
}

template <typename Integer>
void JobManager<Integer>::put(const Row<Integer>& row) const
{
   facets.put(row);
}

template <typename Integer>
Facet<Integer> JobManager<Integer>::get() const
{
   return facets.get();
}

#ifndef MPI_SUPPORT
   #pragma GCC diagnostic push
   #pragma GCC diagnostic ignored "-Wunused-parameter"
   #pragma clang diagnostic push
   #pragma clang diagnostic ignored "-Wunused-parameter"
#endif
template <typename Integer>
JobManager<Integer>::JobManager(const Names& names_, const int number_of_processors, const int threads_per_processor)
:
   communication(),
   facets(names_),
   request_threads() // vital implementation detail: threads may access other members, hence, the threads must be destroyed first (Destruction in reverse order of construction).
{
   #ifdef MPI_SUPPORT
   for ( int id(1); id < number_of_processors; ++id ) // for all nodes except the master
   {
      for ( int i(0); i < threads_per_processor; ++i )
      {
         request_threads.emplace_front([&,id]() // capture id by value, as it is a local variable
         {
            #ifdef BENCHMARK_LOAD_BALANCING
            std::size_t count(0);
            #endif
            while ( true )
            {
               const auto facet = facets.get();
               communication.toSlave(facet, id);
               if ( facet.empty() ) // if facet is empty, the slave will stop working.
               {
                  break;
               }
               #ifdef BENCHMARK_LOAD_BALANCING
               ++count;
               #endif
               const auto results = communication.fromSlave<Integer>(id);
               put(results);
            }
            #ifdef BENCHMARK_LOAD_BALANCING
            std::stringstream stream;
            stream << "Node " << id << ": " << count << "\n";
            std::cerr << stream.str();
            #endif
         });
      }
   }
   #endif // MPI_SUPPORT
}
#ifndef MPI_SUPPORT
   #pragma GCC diagnostic pop
   #pragma clang diagnostic pop
#endif

