
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "message_passing_interface_session.h"

#include <cassert>

#ifdef MPI_SUPPORT
#include "mpi_no_warnings.h"
#endif

MessagePassingInterfaceSession& getMessagePassingInterfaceSession()
{
   // static asserts thread safe initialization. Only this one session will be created.
   static MessagePassingInterfaceSession session;
   return session;
}

bool MessagePassingInterfaceSession::isMaster() const
{
   #ifdef MPI_SUPPORT
   return MPI::COMM_WORLD.Get_rank() == 0;
   #else
   return true;
   #endif
}

std::size_t MessagePassingInterfaceSession::getNumberOfNodes() const
{
   #ifdef MPI_SUPPORT
   assert( MPI::COMM_WORLD.Get_size() > 0 );
   return MPI::COMM_WORLD.Get_size();
   #else
   return 1;
   #endif
}

MessagePassingInterfaceSession::MessagePassingInterfaceSession()
{
   #ifdef MPI_SUPPORT
   MPI::Init();
   #endif
}

MessagePassingInterfaceSession::~MessagePassingInterfaceSession()
{
   #ifdef MPI_SUPPORT
   MPI::Finalize();
   #endif
}

