
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "method_validation.h"

#include <exception>
#include <iostream>

#include "algorithm_polytope_validation.h"
#include "application_name.h"
#include "delayed_action.h"
#include "input.h"
#include "integer_type_selection.h"
#include "polytope.h"

namespace
{
   /// Implementation of polytope validation.
   template <typename Integer>
   struct ValidationImplementationFunctor
   {
      static int call(int, char**);
   };
}

int method::validation(int argc, char** argv)
{
   std::cerr << APP_NAME << " -- polytope validation\n";
   return IntegerTypeSelector<ValidationImplementationFunctor>::select(argc, argv);
}

namespace
{
   template <typename Integer>
   int ValidationImplementationFunctor<Integer>::call(int argc, char** argv)
   {
      Polytope<Integer> polytope;
      try
      {
         const auto action = makeDelayedAction([](){ std::cerr << "Input completed\n"; },
                                               std::chrono::seconds(2));
         polytope = input::polytope<Integer>(argc, argv);
      }
      catch ( const std::exception& e )
      {
         std::cerr << "Exception caught: " << e.what() << "\n";
         return 1;
      }
      catch ( ... )
      {
         std::cerr << "Unknown exception caught while reading files\n";
         return 1;
      }
      try
      {
         algorithm::polytopeValidation(polytope);
         return 0;
      }
      catch ( const std::exception& e )
      {
         std::cerr << "Exception caught: " << e.what() << "\n";
         return 1;
      }
      catch ( ... )
      {
         std::cerr << "Unknown exception caught while validating polytope\n";
         return 1;
      }
   }
}

