
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "testing_gear.h"

#include "algorithm_inequality_operations.h"

namespace
{
   void right_hand_side();
   void distances();
   void furthest_vertex();
   void nearest_vertex();
}

int main()
try
{
   right_hand_side();
   distances();
   furthest_vertex();
   nearest_vertex();
}
catch ( const std::exception& e )
{
   std::cerr << e.what() << "\n";
   return 1;
}

namespace
{
   void right_hand_side()
   {
      ASSERT(rhs(Inequality<int>{1, 0, -1}) == 1, "Bad rhs.");
   }
   void distances()
   {
      ASSERT((distance(Inequality<int>{1, 0, -1}, Vertex<int>{ 0, 0, 1}) ==  1), "Bad distance.");
      ASSERT((distance(Inequality<int>{1, 0, -1}, Vertex<int>{-1, 1, 1}) ==  2), "Bad distance.");
      ASSERT((distance(Inequality<int>{1, 0, -1}, Vertex<int>{ 1, 0, 1}) ==  0), "Bad distance.");
      ASSERT((distance(Inequality<int>{1, 0, -1}, Vertex<int>{ 2, 1, 1}) == -1), "Bad distance.");
   }
   void furthest_vertex()
   {
      Vertices<int> vs{{0, 0, 1}, {0, 1, 1}, {1, 0, 1}, {1, 1, 1}};
      ASSERT((furthestVertex(vs, Inequality<int>{1, 0, -1}) == Vertex<int>{0, 0, 1}) ||
             (nearestVertex(vs, Inequality<int>{1, 0, -1}) == Vertex<int>{0, 1, 1}), "");
   }
   void nearest_vertex()
   {
      Vertices<int> vs{{0, 0, 1}, {0, 1, 1}, {1, 0, 1}, {1, 1, 1}};
      ASSERT((nearestVertex(vs, Inequality<int>{1, 0, -1}) == Vertex<int>{1, 0, 1}) ||
             (nearestVertex(vs, Inequality<int>{1, 0, -1}) == Vertex<int>{1, 1, 1}), "");
   }
}

