
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "testing_gear.h"

#include "safe_integer.h"

#include <limits>

namespace
{
   void test_constructors();
   void test_operator_int();
}

int main()
try
{
   test_constructors();
   test_operator_int();
}
catch ( const std::exception& e )
{
   std::cerr << e.what() << "\n";
   return 1;
}

namespace
{
   using SI = SafeInteger;
   using SID = SafeInteger::DataType;

   void test_constructors()
   {
      ASSERT_NOTHROW(SI(std::numeric_limits<SID>::min()), "minimal value must be valid.");
      ASSERT_NOTHROW(SI(std::numeric_limits<SID>::max()), "maximal value must be valid.");
      ASSERT_ANY_EXCEPTION(SI(static_cast<uint64_t>(std::numeric_limits<SID>::max()) + 1), "Argument is definitively too large.");
   }

   void test_operator_int()
   {
      ASSERT(static_cast<int>(SI(10)) == 10, "operator int()");
      ASSERT(static_cast<int>(SI(-10)) == -10, "operator int()");
      const auto max = std::numeric_limits<int>::max();
      const auto min = std::numeric_limits<int>::min();
      ASSERT(static_cast<int>(SI(max)) == max, "operator int()");
      ASSERT(static_cast<int>(SI(min)) == min, "operator int()");
   }
}

